# Python imports
import json
import logging
import sys

# Django imports
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt

# Models imports
from api.models.pipedream_model import GPhLPipedream
from api.models.report_model import Report
from api.models.dpstep_model import DPStep
from api.models.ligandsfitting_model import LigandsFitting
from api.models.reductionscaling_model import ReductionScaling
from api.models.refinement_model import Refinement
from api.models.postrefinement_model import PostRefinement

# Activate logger
logger=logging.getLogger('dataproc')

@csrf_exempt
def storeProcInput(request):

    """
    Parse JSON data and call store functions for each model
    """

    if request.method=='POST':
        json_data=json.loads(request.body)

        try:
            # Register nodes
            storeParseReport(json_data)

            # Register relationships 
            # connectConstructUser(json_data_construct, json_data_user)

            return JsonResponse({"STATUS": "INPUT SUCCESSFULLY REGISTERED"})
        
        except :
            return JsonResponse({"STATUS":"ERROR OCCURRED"}, safe=False)

# @csrf_exempt
# def storeParseGPhLPipedream(data):

#     """
#     Creates nodes for each # with relative properties
#     """

#     try:
#         gphl_pipedream=GPhLPipedream(command=data['command'], 
#             jsonversion=data['jsonversion'],
#             runby=data['runby'],
#             runfrom=data['runfrom'],
#             jobid=data['jobid'],
#             output=data['output'],
#             version=data['version'])
#         gphl_pipedream.save()
#         return gphl_pipedream.serialize
    
#     except:
#         print(sys.exc_info()[0])
#         return ({"STATUS": "ERROR OCCURRED WHILE REGISTERING DATASET"})

@csrf_exempt
def storeParseReport(data):

    """
    Creates nodes for each report with relative properties
    """

    try:
        logger.info(data)
        
        GPhL_pipedream=data['GPhL_pipedream']
        ligandsstatistics=data['ligandfitting']['ligands']['1']['validationstatistics']['ligandstatistics']['1']
        molprobity=data['ligandfitting']['ligands']['1']['validationstatistics']['molprobity']
        solutions=data['ligandfitting']['ligands']['1']['solutions']['1']

        autoprocscaling=data['dataprocessing']['processingdata']['AutoProcscaling']
        autoprocscalingstatistics=data['dataprocessing']['processingdata']['AutoProcScalingStatistics']
        autoproc=data['dataprocessing']['processingdata']['AutoProc']
        referencedata=data['dataprocessing']['referencedata']

        # # postrefinement=data['ligandfitting']['ligands']['1']['postrefinement']['cycles']['2']

        report=Report(command=GPhL_pipedream['command'], 
            jsonversion=GPhL_pipedream['jsonversion'],
            runby=GPhL_pipedream['runby'],
            runfrom=GPhL_pipedream['runfrom'],
            jobid=GPhL_pipedream['jobid'],
            gphlpipedream_output=GPhL_pipedream['output'],
            version=GPhL_pipedream['version'],

            ligandomin=ligandsstatistics['ligandomin'],
            mogulzbond=ligandsstatistics['mogulzbond'],
            ligandbmin=ligandsstatistics['ligandbmin'],
            mogulring=ligandsstatistics['mogulring'],
            moguldihe=ligandsstatistics['moguldihe'],
            ligandbavg=ligandsstatistics['ligandbavg'],
            mogulangl=ligandsstatistics['mogulangl'],
            mogulbond=ligandsstatistics['mogulbond'],
            ligandbmax=ligandsstatistics['ligandbmax'],
            ligandid=ligandsstatistics['ligandid'],
            ligandomax=ligandsstatistics['ligandomax'],
            ligandcc=ligandsstatistics['ligandcc'],
            mogulzangl=ligandsstatistics['mogulzangl'],
            
            molprobitypercentile=molprobity['molprobitypercentile'],
            ramaoutlierpercent=molprobity['ramaoutlierpercent'],
            cbetadeviations=molprobity['cbetadeviations'],
            ramafavoredpercent=molprobity['ramafavoredpercent'],
            poorrotamers=molprobity['poorrotamers'],
            rmsbonds=molprobity['rmsbonds'],
            rmsangles=molprobity['rmsangles'],
            clashpercentile=molprobity['clashpercentile'],
            poorrotamerspercent=molprobity['poorrotamerspercent'],
            clashscore=molprobity['clashscore'],
            ramafavored=molprobity['ramafavored'],
            molprobityscore=molprobity['molprobityscore'],
            ramaoutliers=molprobity['ramaoutliers'],
            
            losecontacts=solutions['closecontacts'],
            chain=solutions['chain'],
            contactscore=solutions['contactscore'],
            ligandstrain=solutions['ligandstrain'],
            poorfit=solutions['poorfit'],
            output=solutions['output'],
            correlationcoefficient=solutions['correlationcoefficient'],
            rhofitscore=solutions['rhofitscore'],

            recordTimeStamp=autoprocscaling['recordTimeStamp'],
            resolutionEllipsoidAxis13=autoprocscaling['resolutionEllipsoidAxis13'],
            resolutionEllipsoidAxis33=autoprocscaling['resolutionEllipsoidAxis33'],
            resolutionEllipsoidAxis23=autoprocscaling['resolutionEllipsoidAxis23'],
            resolutionEllipsoidValue2=autoprocscaling['resolutionEllipsoidValue2'],
            resolutionEllipsoidAxis21=autoprocscaling['resolutionEllipsoidAxis21'],
            resolutionEllipsoidAxis31=autoprocscaling['resolutionEllipsoidAxis31'],
            resolutionEllipsoidAxis22=autoprocscaling['resolutionEllipsoidAxis22'],
            resolutionEllipsoidAxis32=autoprocscaling['resolutionEllipsoidAxis32'],
            resolutionEllipsoidValue3=autoprocscaling['resolutionEllipsoidValue3'],
            resolutionEllipsoidAxis12=autoprocscaling['resolutionEllipsoidAxis12'],
            resolutionEllipsoidValue1=autoprocscaling['resolutionEllipsoidValue1'],
            resolutionEllipsoidAxis11=autoprocscaling['resolutionEllipsoidAxis11'],

            innerShell_multiplicity=autoprocscalingstatistics['innerShell']['multiplicity'],
            innerShell_resolutionLimitLow=autoprocscalingstatistics['innerShell']['resolutionLimitLow'],
            innerShell_rMeasWithinIPlusIMinus=autoprocscalingstatistics['innerShell']['rMeasWithinIPlusIMinus'],
            innerShell_DanoOverSigDano=autoprocscalingstatistics['innerShell']['DanoOverSigDano'],
            innerShell_completenessSpherical=autoprocscalingstatistics['innerShell']['completenessSpherical'],
            innerShell_nTotalUniqueObservations=autoprocscalingstatistics['innerShell']['nTotalUniqueObservations'],
            innerShell_rMerge=autoprocscalingstatistics['innerShell']['rMerge'],
            innerShell_rPimWithinIPlusIMinus=autoprocscalingstatistics['innerShell']['rPimWithinIPlusIMinus'],
            innerShell_anomalousCompletenessEllipsoidal=autoprocscalingstatistics['innerShell']['anomalousCompletenessEllipsoidal'],
            innerShell_nTotalObservations=autoprocscalingstatistics['innerShell']['nTotalObservations'],
            innerShell_completenessEllipsoidal=autoprocscalingstatistics['innerShell']['completenessEllipsoidal'],
            innerShell_anomalousCompleteness=autoprocscalingstatistics['innerShell']['anomalousCompleteness'],
            innerShell_anomalousMultiplicity=autoprocscalingstatistics['innerShell']['anomalousMultiplicity'],
            innerShell_resolutionLimitHigh=autoprocscalingstatistics['innerShell']['resolutionLimitHigh'],
            innerShell_completeness=autoprocscalingstatistics['innerShell']['completeness'],
            innerShell_ccHalf=autoprocscalingstatistics['innerShell']['ccHalf'],
            innerShell_rPimAllIPlusIMinus=autoprocscalingstatistics['innerShell']['rPimAllIPlusIMinus'],
            innerShell_meanIOverSigI=autoprocscalingstatistics['innerShell']['meanIOverSigI'],
            innerShell_anomalousCompletenessSpherical=autoprocscalingstatistics['innerShell']['anomalousCompletenessSpherical'],
            innerShell_ccAnomalous=autoprocscalingstatistics['innerShell']['ccAnomalous'],
            innerShell_rMeasAllIPlusIMinus=autoprocscalingstatistics['innerShell']['rMeasAllIPlusIMinus'],

            outerShell_multiplicity=autoprocscalingstatistics['outerShell']['multiplicity'],
            outerShell_resolutionLimitLow=autoprocscalingstatistics['outerShell']['resolutionLimitLow'],
            outerShell_rMeasWithinIPlusIMinus=autoprocscalingstatistics['outerShell']['rMeasWithinIPlusIMinus'],
            outerShell_DanoOverSigDano=autoprocscalingstatistics['outerShell']['DanoOverSigDano'],
            outerShell_completenessSpherical=autoprocscalingstatistics['outerShell']['completenessSpherical'],
            outerShell_nTotalUniqueObservations=autoprocscalingstatistics['outerShell']['nTotalUniqueObservations'],
            outerShell_rMerge=autoprocscalingstatistics['outerShell']['rMerge'],
            outerShell_rPimWithinIPlusIMinus=autoprocscalingstatistics['outerShell']['rPimWithinIPlusIMinus'],
            outerShell_anomalousCompletenessEllipsoidal=autoprocscalingstatistics['outerShell']['anomalousCompletenessEllipsoidal'],
            outerShell_nTotalObservations=autoprocscalingstatistics['outerShell']['nTotalObservations'],
            outerShell_completenessEllipsoidal=autoprocscalingstatistics['outerShell']['completenessEllipsoidal'],
            outerShell_anomalousCompleteness=autoprocscalingstatistics['outerShell']['anomalousCompleteness'],
            outerShell_anomalousMultiplicity=autoprocscalingstatistics['outerShell']['anomalousMultiplicity'],
            outerShell_resolutionLimitHigh=autoprocscalingstatistics['outerShell']['resolutionLimitHigh'],
            outerShell_completeness=autoprocscalingstatistics['outerShell']['completeness'],
            outerShell_ccHalf=autoprocscalingstatistics['outerShell']['ccHalf'],
            outerShell_rPimAllIPlusIMinus=autoprocscalingstatistics['outerShell']['rPimAllIPlusIMinus'],
            outerShell_meanIOverSigI=autoprocscalingstatistics['outerShell']['meanIOverSigI'],
            outerShell_anomalousCompletenessSpherical=autoprocscalingstatistics['outerShell']['anomalousCompletenessSpherical'],
            outerShell_ccAnomalous=autoprocscalingstatistics['outerShell']['ccAnomalous'],
            outerShell_rMeasAllIPlusIMinus=autoprocscalingstatistics['outerShell']['rMeasAllIPlusIMinus'],

            overall_multiplicity=autoprocscalingstatistics['overall']['multiplicity'],
            overall_resolutionLimitLow=autoprocscalingstatistics['overall']['resolutionLimitLow'],
            overall_rMeasWithinIPlusIMinus=autoprocscalingstatistics['overall']['rMeasWithinIPlusIMinus'],
            overall_DanoOverSigDano=autoprocscalingstatistics['overall']['DanoOverSigDano'],
            overall_completenessSpherical=autoprocscalingstatistics['overall']['completenessSpherical'],
            overall_nTotalUniqueObservations=autoprocscalingstatistics['overall']['nTotalUniqueObservations'],
            overall_rMerge=autoprocscalingstatistics['overall']['rMerge'],
            overall_rPimWithinIPlusIMinus=autoprocscalingstatistics['overall']['rPimWithinIPlusIMinus'],
            overall_anomalousCompletenessEllipsoidal=autoprocscalingstatistics['overall']['anomalousCompletenessEllipsoidal'],
            overall_nTotalObservations=autoprocscalingstatistics['overall']['nTotalObservations'],
            overall_completenessEllipsoidal=autoprocscalingstatistics['overall']['completenessEllipsoidal'],
            overall_anomalousCompleteness=autoprocscalingstatistics['overall']['anomalousCompleteness'],
            overall_anomalousMultiplicity=autoprocscalingstatistics['overall']['anomalousMultiplicity'],
            overall_resolutionLimitHigh=autoprocscalingstatistics['overall']['resolutionLimitHigh'],
            overall_completeness=autoprocscalingstatistics['overall']['completeness'],
            overall_ccHalf=autoprocscalingstatistics['overall']['ccHalf'],
            overall_rPimAllIPlusIMinus=autoprocscalingstatistics['overall']['rPimAllIPlusIMinus'],
            overall_meanIOverSigI=autoprocscalingstatistics['overall']['meanIOverSigI'],
            overall_anomalousCompletenessSpherical=autoprocscalingstatistics['overall']['anomalousCompletenessSpherical'],
            overall_ccAnomalous=autoprocscalingstatistics['overall']['ccAnomalous'],
            overall_rMeasAllIPlusIMinus=autoprocscalingstatistics['overall']['rMeasAllIPlusIMinus'],

            datatype=data['dataprocessing']['datatype'],
            processingtype=data['dataprocessing']['processingtype'],
            symmetry=referencedata['symmetry'],
            gamma=referencedata['cell']['gamma'],
            a=referencedata['cell']['a'],
            alpha=referencedata['cell']['alpha'],
            b=referencedata['cell']['b'],
            beta=referencedata['cell']['beta'],
            c=referencedata['cell']['c'],

        #     refinedCell_beta=autoproc['refinedCell_beta'],
        #     refinedCell_b=autoproc['refinedCell_b'],
        #     wavelength=autoproc['wavelength'],
        #     refinedCell_a=autoproc['refinedCell_a'],
        #     refinedCell_alpha=autoproc['refinedCell_alpha'],
        #     spaceGroup=autoproc['spaceGroup'],
        #     refinedCell_c=autoproc['refinedCell_c'],
        #     refinedCell_gamma=autoproc['refinedCell_gamma'],

        number=data['refinement']['models']['1']['number'],
        LMRscore=data['refinement']['models']['1']['LMRscore'],
        modelname=data['refinement']['models']['1']['modelname'],
        selectedmodel=data['refinement']['selectedmodel'],
        refinementprotocol=data['refinement']['refinementprotocol'])

            # postrefinement_WilsonB=['WilsonB'],
            # postrefinement_Rfree=['Rfree'],
            # postrefinement_MeanB=['MeanB'],
            # postrefinement_type=['type'],
            # postrefinement_step=['step'],
            # postrefinement_RMSbonds=['RMSbonds'],
            # postrefinement_RMSangles=['RMSangles'],
            # postrefinement_R=['R'],
            # postrefinement_WatersPresent=['WatersPresent'],

            # refinement_WilsonB=['WilsonB'],
            # refinement_Rfree=['Rfree'],
            # refinement_MeanB=['MeanB'],
            # refinement_type=['type'],
            # refinement_step=['step'],
            # refinement_RMSbonds=['RMSbonds'],
            # refinement_RMSangles=['RMSangles'],
            # refinement_R=['R'],
            # refinement_WatersPresent=['WatersPresent'],

        report.save()
        logger.info('SAVE')
        return report.serialize
    
    except:
        print(sys.exc_info()[0])
        return ({"STATUS": "ERROR OCCURRED WHILE REGISTERING REPORT"})

# @csrf_exempt
# def connectDPStepReport(data1, data2):

#     """
#     Create a relationship between a dpstep and a report
#     """
    
#     try:
#         dpstep=DPStep.nodes.get(name=data1["uuid"])
#         report=Report.nodes.get(uuid=data2["name"])
#         return JsonResponse({"STATUS": dpstep.generates_report.connect(report)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO REPORT"}, safe=False)

# @csrf_exempt
# def connectDPStepRefinement(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a refinement node
    
#     try:
#         dpstep=DPStep.nodes.get(name=data1["uuid"])
#         refinement=Refinement.nodes.get(uuid=data2["uuid"])
#         return JsonResponse({"STATUS": dpstep.is_refinement.connect(refinement)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO REFINEMENT"}, safe=False)

# @csrf_exempt
# def connectDPStepPostRefinement(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a postrefinement node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         postrefinment=PostRefinement.nodes.get(uuid=data2["uuid"])
#         return JsonResponse({"STATUS": dpstep.is_postrefinement.connect(postrefinement)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO POST-REFINEMENT"}, safe=False)

# @csrf_exempt
# def connectDPStepLigandsFitting(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a ligandsfitting node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         ligandsfitting=LigandsFitting.nodes.get(uuid=data2["uuid"])
#         return JsonResponse({"STATUS": dataset.belongs.connect(construct)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO LIGANDS FITTING NODE"}, safe=False)

# @csrf_exempt
# def connectDPStepReductionScaling(data1, data2):

#     """
#     Create a relationship between a dpstep and a reductionscaling node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         reductionscaling=StorageHost.nodes.get(uuid=data2["uuid"])
#         return JsonResponse({"Status": dpstep.is_reductionscaling.connect(reductionscaling)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO REDUCTION SACLING NODE"}, safe=False)

# @csrf_exempt
# def connectDPStepautoPROC(data1, data2):
    
#     """
#     Create a relationship between a dpstep and an autoRPOC node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         autoproc=autoPROC.nodes.get(uuid=data2["uuid"])
#         return JsonResponse({"STATUS": dpstep.with_autoproc.connect(autoproc)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO AUTOPROC NODE"}, safe=False)

# @csrf_exempt
# def connectDPStepRhofit(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a rhofit node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         rhofit=Rhofit.nodes.get(uuid=data2["uuid"])

#         return JsonResponse({"STATUS": dpstep.with_rhofit.connect(rhofit)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO RHOFIT NODE"}, safe=False)

# @csrf_exempt
# def connectDPStepBuster(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a buster node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         buster=Buster.nodes.get(uuid=data2["uuid"])

#         return JsonResponse({"STATUS": dpstep.wiht_buster.connect(buster)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO BUSTER NODE"}, safe=False)

# @csrf_exempt
# def connectDPStepPipedream(data1, data2):
    
#     """
#     Create a relationship between a dpstep and a pipedream node
#     """

#     try:
#         dpstep=DPStep.nodes.get(uuid=data1["uuid"])
#         pipedream=GPhLPipedream.nodes.get(uuid=data2["uuid"])

#         return JsonResponse({"STATUS": dpstep.with_pipedream.connect(pipedream)}, safe=False)

#     except:
#         return JsonResponse({"STATUS": "ERROR OCCURRED WHILE CONNECTING DPSTEP TO PIPEDREAM NODE"}, safe=False)

